/***************************************************************************
 *   Copyright (C) 2004 by Hans Oischinger                                 *
 *   hans.oischinger@kdemail.net                                           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.             *
 ***************************************************************************/
#include "animatedtransformlist.h"

#include "transformlist.h"

// Visitors
#include "visitors/avisitor.h"

AnimatedTransformList::AnimatedTransformList( TransformList *list)
 : Attribute(ATTR_TRANSFORM)
{
  if (list)
    m_baseVal = list;
  else
    m_baseVal = new TransformList();
  m_animVal = new TransformList();
}

AnimatedTransformList::AnimatedTransformList(const AnimatedTransformList &other) : Attribute(ATTR_TRANSFORM)
{
  (*this) = other;
}

AnimatedTransformList::~AnimatedTransformList()
{
  delete m_baseVal;
  delete m_animVal;
}

AnimatedTransformList &AnimatedTransformList::operator=(const AnimatedTransformList &other)
{
  Attribute::operator=(other);
  delete m_baseVal;
  delete m_animVal;
  m_baseVal = new TransformList(other.baseVal());
  m_animVal = new TransformList(other.animVal());

  return *this;
}

Attribute* AnimatedTransformList::clone()
{
  AnimatedTransformList* retElem = new AnimatedTransformList();
  *retElem = *this;
  return ( retElem );
}

/**
 * Overrides default isValid() method and delegate request to baseVal
 * @return 
 */
bool AnimatedTransformList::isValid( ) const
{
  return m_baseVal->isValid();
}

void AnimatedTransformList::accept( AVisitor &a )
{
  a.visit(*this);
}
